#define _GNU_SOURCE
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

#define TERMINATE_SHM "/terminate"
#define TERMINATE_SHM_SIZE 8

void* open_shm(char *shm_name, size_t shm_size) {
    int shmfd = shm_open(shm_name, O_RDWR | O_CREAT, 0600);
    if(shmfd == -1) {
        fprintf(stderr, "ERROR: Failed creating shared memory object\n");
        exit(EXIT_FAILURE);
    }

    if(ftruncate(shmfd, shm_size) < 0) {
        fprintf(stderr, "ERROR: Failed truncating shared memory object\n");
        exit(EXIT_FAILURE);
    }

    void* shm;
    shm = mmap(NULL, shm_size, PROT_READ | PROT_WRITE, MAP_SHARED, shmfd, 0);

    if(shm == MAP_FAILED) {
        fprintf(stderr, "ERROR: Failed mapping shared memory object\n");
        exit(EXIT_FAILURE);
    }

    return shm;
}

void write_to_shm(void* shm, char *message, size_t message_size) {
    memcpy(shm, message, message_size);
}

void close_shm(void* shm, size_t shm_size) {
    if (munmap(shm, shm_size) == -1) {
        fprintf(stderr, "ERROR: Failed unmapping shared memory object\n");
        exit(EXIT_FAILURE);
    }
}

void destroy_shm(char *shm_name) {
    if (shm_unlink(shm_name) == -1) {
        fprintf(stderr, "ERROR: Failed unlinking shared memory object\n");
        exit(EXIT_FAILURE);
    }
}
